<?php
/**
 *
 * @category        snippets
 * @package         minified
 * @author          Ruud Eisinga - Dev4me
 * @link			http://www.dev4me.nl/
 * @license         http://www.gnu.org/licenses/gpl.html
 * @platform        WebsiteBaker 2.8.x
 * @requirements    PHP 7.2 and higher
 * @version         1.7.3
 * @lastmodified    March 15, 2024
 *
 */

if(defined('WB_PATH') == false) { exit("Cannot access this file directly"); }
define ('MINIFIED_DEBUG',false);

$path = dirname(__FILE__).'/lib';
require_once $path . '/minify/src/Minify.php';
require_once $path . '/minify/src/CSS.php';
require_once $path . '/minify/src/JS.php';
require_once $path . '/minify/src/Exception.php';
require_once $path . '/minify/src/Exceptions/BasicException.php';
require_once $path . '/minify/src/Exceptions/FileImportException.php';
require_once $path . '/minify/src/Exceptions/IOException.php';
require_once $path . '/path-converter/src/ConverterInterface.php';
require_once $path . '/path-converter/src/Converter.php';

use MatthiasMullie\Minify;

global $css, $js;

function minify_css($add_frontend = true) {
	global $css;
	$key = _get_minify_key('css');
	if ($add_frontend) _get_frontend('css');
	if(_need_minify('css')) {
		$key = _get_minify_key('css',true);
		$css_minifier = new Minify\CSS('');
		foreach ( $css as $cssfile ) {
			$css_minifier->add(_minify_filename($cssfile, true,"Add CSS"));
		}
		$output = _minify_filename('minified-'.$key.'.css',false,"Create CSS");
		$css_minifier->minify($output);
	} else {
		_do_log("No CSS changes");
	}
	echo '<link href="'.TEMPLATE_DIR.'/minified-'.$key.'.css" rel="stylesheet" />'.PHP_EOL;
} 

function minify_js($add_frontend = true) {
	global $js;
	$key = _get_minify_key('js');
	if ($add_frontend) _get_frontend('js');
	if(_need_minify('js')) {
		$key = _get_minify_key('js',true);
		$js_minifier = new Minify\JS('');
		foreach ( $js as $jsfile ) {
			$js_minifier->add(_minify_filename($jsfile,true,"Add JS"));
		}
		$output = _minify_filename('minified-'.$key.'.js',false,"Create JS");
		$js_minifier->minify($output);
	} else {
		_do_log("No JS changes");
	}
	echo '<script src="'. TEMPLATE_DIR .'/minified-'.$key.'.js"></script>'.PHP_EOL;
} 

function _minify_filename($relative_file, $get_content = true, $caller = '') {
	if($caller) _do_log($caller." > relative: ".$relative_file);
	$path = _clean_filename(WB_PATH);
	if(substr($relative_file,0,2) == "//") $relative_file = 'https:'.$relative_file;
	if(substr($relative_file,0,5) == "http:" || substr($relative_file,0,6) == "https:") {
		if($get_content) {
			if($caller) _do_log($caller." - loading external: ".$relative_file);
			return file_get_contents($relative_file);
		}
			if($caller) _do_log($caller." - No check possible: ".$relative_file);
		return null;
	}	
	$filename = null;
	if(substr($relative_file,0,1) != "/") { 		// relative to template dir
		$template_path = str_ireplace(WB_URL,$path,TEMPLATE_DIR);
		$filename = $template_path.'/'.$relative_file;
	} elseif (file_exists( $path.$relative_file )) {  // relative to WB root dir
		$filename = $path.$relative_file;
	} elseif (file_exists( $relative_file )) {  		// direct file
		$filename = $relative_file;
	}
	$filename = _clean_filename($filename);

	if($caller) _do_log($caller." < using: ".$filename);
	return $filename;
}

function _need_minify($type = 'css') {
	global $css, $js;
	$key = _get_minify_key($type);
	if(!isset($_SESSION['USER_ID'])) return false;  //rebuild only when logged in user
	if(isset($_SESSION['USER_ID'])) {  //Detect CTRL-F5 or ?force parameter only when logged in user
		if(isset($_GET['force'])) {
			_do_log("Force parameter detected, rebuild $type");
			return true;
		}
		if(isset($_SERVER['HTTP_CACHE_CONTROL']) && $_SERVER['HTTP_CACHE_CONTROL'] === 'no-cache') {
			_do_log("CTRL-F5 detected, rebuild $type");
			return true;
		}
	}
	if($type == 'css') {
		$ofile = filemtime(_minify_filename('minified-'.$key.'.css',false));
		if(!$ofile) return true;
		foreach ( $css as $cssfile ) {
			if($thefile = _minify_filename($cssfile,false,"Check timestamp")) {
				if(!file_exists($thefile)) {
					_do_log("File does not exist: ".$thefile);
				}elseif(filemtime($thefile) > $ofile) {
					_do_log("Changed file: ".$thefile);
					return true;
				}else{
					_do_log("Not changed: ".$thefile);
				}
			}
		}
	} elseif($type == 'js') {
		$ofile = filemtime(_minify_filename('minified-'.$key.'.js',false));
		if(!$ofile) return true;
		foreach ( $js as $jsfile ) {
			if($thefile = _minify_filename($jsfile,false,"Check timestamp")) {
				if(!file_exists($thefile)) {
					_do_log("File does not exist: ".$thefile);
				}elseif(filemtime($thefile) > $ofile) {
					_do_log("Changed file: ".$thefile);
					return true;
				}else{
					_do_log("Not changed: ".$thefile);
				}			
			}
		}

	}
	return false;
	
}
function _get_frontend($type= 'css') {
	global $database,$css,$js;
	$path = _clean_filename(WB_PATH);
	$query = "SELECT DISTINCT `directory` FROM ".TABLE_PREFIX."addons where function='snippet'";
	$res = $database->query($query);
	if($res && $res->numRows() > 0) {
		while ($row = $res->fetchRow()) {
			$fpath = $path.'/modules/'.$row['directory'];
			if($type=='css' && file_exists($fpath.'/frontend.css')) $css[] = $fpath.'/frontend.css';
			if($type=='js' && file_exists($fpath.'/frontend.js')) $js[] = $fpath.'/frontend.js';
		}
	}	
	$query = "SELECT DISTINCT `directory` FROM ".TABLE_PREFIX."addons a JOIN
			 ".TABLE_PREFIX."sections s ON s.module = a.directory";
	$res = $database->query($query);
	if($res && $res->numRows() > 0) {
		while ($row = $res->fetchRow()) {
			$fpath = $path.'/modules/'.$row['directory'];
			if($type=='css' && file_exists($fpath.'/frontend.css')) $css[] = $fpath.'/frontend.css';
			if($type=='js' && file_exists($fpath.'/frontend.js')) $js[] = $fpath.'/frontend.js';
		}
	}
	$done = 1;
}

function _clean_filename($filename) {
	if(preg_match('/^[A-Z]:/i', $filename)) {  		// detect windows absolute path
		$filename = substr($filename,2);  			// strip drive letter
	}
	$filename = str_replace('\\','/',$filename);	// change windows \ into /
	return $filename;
}

function _do_log($message) {
	if(!MINIFIED_DEBUG) return;
    $sLogfile = ini_get ('error_log');
    if (!is_writeable($sLogfile)){return false;}
	$sMsg = date('c').' '.'[MINIFY] "'.$message.'"'.PHP_EOL;
    file_put_contents($sLogfile, $sMsg, FILE_APPEND);
}

function _get_minify_key($type = 'css', $renew = false) {
	global $database;
	$templ = defined('TEMPLATE') ? TEMPLATE : 'base';
	$key = $database->get_one("SELECT `value` FROM `".TABLE_PREFIX."settings` WHERE `name`='minify_key_{$type}_{$templ}' LIMIT 1");
	if(!$key || $renew) {
		$path = str_ireplace(WB_URL,WB_PATH,TEMPLATE_DIR);
		$path = str_replace('\\','/',$path);
		if(file_exists($path.'/minified-'.$key.'.css')) unlink($path.'/minified-'.$key.'.css');
		if(file_exists($path.'/minified-'.$key.'.js')) unlink($path.'/minified-'.$key.'.js');
		$newkey = md5(microtime());
		if(!$key) {
			$database->query("INSERT INTO `".TABLE_PREFIX."settings` (`name`, `value`) VALUES ('minify_key_{$type}_{$templ}', '$newkey')");
		} else {
			$database->query("UPDATE `".TABLE_PREFIX."settings` SET `value` = '$newkey' WHERE `name` = 'minify_key_{$type}_{$templ}'");
		}
		$key = $newkey;
	}
	return $key;
}